#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include <timer.h>
#include "global.h"
#include "game.h"
#include "choose.h"
#include "exist.h"
#include "requesters.h"
#include "graphics.h"
#include "utils.h"
#include "messages.h"
#include "pause.h"
#include "ints.h"
#include "wav_delete.h"
#include "wav_elapsed.h"
#include "wav_insert.h"
#include "wav_inexists.h"



struct hall_fame current_hall_fame = { HALLFAMEMAGICNUMBER, 0, 0, 0, 0 } ;



void StartNewGame (void)
{
  u8 i, j ;

  current_game.current_turn = 0 ;
  if (!RandomChoose (current_game.solution)) {
    current_game.current_turn = -1 ;
    return ;
  }

  for (i = 0; i < NUMBER_OF_TURNS; i++) {
    for (j = 0; j < WORD_LEN; j++)
      current_game.colors_history[i][j] = WRONG_LETTER ;
    strcpy (current_game.suggestions_history[i], BLANK_STRING_INIT) ;
  }

  // Set the first line's color and starting/ending letters, that is the
  // initial hint, and update the currently known facts (i.e the currently
  // known letters at the right place.
  strcpy (current_game.current_known_facts, BLANK_STRING_INIT) ;
  current_game.current_known_facts[0] = current_game.solution[0] ;
  current_game.current_known_facts[WORD_LEN-1] =
    current_game.solution[WORD_LEN-1] ;
  strcpy (current_game.suggestions_history[0],
	  current_game.current_known_facts) ;
  current_game.colors_history[0][0] = GOOD_PLACE ;
  current_game.colors_history[0][WORD_LEN-1] = GOOD_PLACE ;
  
  /* Comme on a mis la premire lettre, c'est comme si */
  /* l'utilisateur avait dj saisi une lettre.        */
  current_game.next_empty_cell = 1 ;
  /* Force le dessin du tableau de jeu. */
  refresh_board () ;
  clear_timer_frame () ;
  start_game_timer () ;
}



void TerminateGame (unsigned char status)
{
  stop_game_timer () ;
  current_game.current_turn = -1 ;
  switch (status) {
  case WINNER:
    current_hall_fame.nb_won++ ;
    break ;

  case LOSER:
    current_hall_fame.nb_lost++ ;
    break ;
    
  case SURRENDER:
    current_hall_fame.nb_surrendered++ ;
    break ;

  default:
    notify_long_message (FALSE, "TerminateGame\nASSERT 1") ;
    break ;
  }
}




void SaveGame (void)
{
  u8 *save_memory = (u8*) 0x0E000000 ;
  u8 *src ;
  int i ;

  if (notify_long_message (TRUE, messages[MSG_SAVE_STATS][language])) {
    current_hall_fame.rand_new_seed = ((int) rand ()) + REG_VCOUNT ;
    src = (char*) &current_hall_fame ;
    for (i=0; i<sizeof (struct hall_fame); i++)
      save_memory[i] = src[i] ;
    notify_long_message (FALSE, messages[MSG_STATS_SAVED][language]) ;
  }
}



void ClearStats (void)
{
  u8 *save_memory = (u8*) 0x0E000000 ;
  u8 *src ;
  int i ;

  if (notify_long_message (TRUE, messages[MSG_CLEAR_STATS][language])) {
    current_hall_fame.rand_new_seed = ((int) rand ()) + REG_VCOUNT ;
    current_hall_fame.nb_won = 0 ;
    current_hall_fame.nb_lost = 0 ;
    current_hall_fame.nb_surrendered = 0 ;
    src = (char*) &current_hall_fame ;
    for (i=0; i<sizeof (struct hall_fame); i++)
      save_memory[i] = src[i] ;
    notify_long_message (FALSE, messages[MSG_STATS_CLEARED][language]) ;
  }
}




BOOL LoadGame ()
{
  u8 *save_memory = (u8*) 0x0E000000 ;
  struct hall_fame read_data ;
  u8 *dest ;
  int i ;

  dest = (char*) &read_data ;
  for (i=0; i<sizeof (struct hall_fame); i++)
    dest[i] =  save_memory[i] ;
    
  if (read_data.magic != HALLFAMEMAGICNUMBER) return (FALSE) ;
  else {
    current_hall_fame.nb_won = read_data.nb_won ;
    current_hall_fame.nb_lost = read_data.nb_lost ;
    current_hall_fame.nb_surrendered = read_data.nb_surrendered ;
    srand (read_data.rand_new_seed + REG_VCOUNT + rand ()) ;
    return (TRUE) ;
  }
}


void GetStats (void)
{
  unsigned char tmp_num_buff[12] ;

  strcpy (all_purpose_message_buffer, messages[MSG_VICTORIES][language]) ;
  itoa (current_hall_fame.nb_won, tmp_num_buff) ;
  strcat (all_purpose_message_buffer, tmp_num_buff) ;
  strcat (all_purpose_message_buffer, messages[MSG_DEFEATS][language]) ;
  itoa (current_hall_fame.nb_lost, tmp_num_buff) ;
  strcat (all_purpose_message_buffer, tmp_num_buff) ;
  strcat (all_purpose_message_buffer, messages[MSG_SURRENDER][language]) ;
  itoa (current_hall_fame.nb_surrendered, tmp_num_buff) ;
  strcat (all_purpose_message_buffer, tmp_num_buff) ;
  notify_long_message (FALSE, all_purpose_message_buffer) ;
}



void check_proposition (BOOL elapsed_time)
{
  BOOL seen[WORD_LEN] ;
  char place[WORD_LEN] ;
  int i, j ;

  /* La suggestion courante est dj dans l'historique. */
  /* Vrifions si le jeu est termin. */
  if (strcmp (current_game.suggestions_history[current_game.current_turn],
	      current_game.solution) == 0) {
    /* Gagn. */
    notify_long_message (FALSE, messages[MSG_WON][language]) ;
    /* On colorie juste les lettres en vert pour montrer */
    /* qu'elles avaient bien t devines.               */
    for (i = 0; i < WORD_LEN; i++) {
      current_game.colors_history[current_game.current_turn][i] =
	GOOD_PLACE ;
      current_game.suggestions_history[current_game.current_turn][i] =
	current_game.solution[i] ;
    }
    refresh_board () ;
    TerminateGame (WINNER) ;
    return ;
  }

  if (elapsed_time) {
    strcpy (current_game.suggestions_history[current_game.current_turn],
	    "      ") ;
    notify_long_message (FALSE, messages[MSG_ELAPSED_TIME][language]) ;
  }

  /* Perdu. */
  if (current_game.current_turn == (NUMBER_OF_TURNS - 1)) {
    strcpy (all_purpose_message_buffer, messages[MSG_LOST][language]) ;
    strcat (all_purpose_message_buffer, current_game.solution) ;
    notify_long_message (FALSE, all_purpose_message_buffer) ;
    TerminateGame (LOSER) ;
    return ;
  }

  /* Initialisation du processus de vrification. */
  for (i = 0; i < WORD_LEN; i++) {
    seen[i] = FALSE ;
    place[i] = WRONG_LETTER ;
  }

  /* Si le temps imparti est dpass, le joueur a perdu son tour. */
  /* Pas de message ici, il a dj t prvenu plus haut.         */
  if (elapsed_time) goto lose_turn ;

  // First check if the player want to play hard by losing his turn
  // if his suggestion is not a valid word inside the dictionary.
  if (check_flag) {
    /* Si la suggestion n'existe pas, le joueur a perdu son tour. */
    if (!Search
	((unsigned char*)
	 current_game.suggestions_history[current_game.current_turn])) {
      play_sound (g_wav_inexists) ;
      strcpy (current_game.suggestions_history[current_game.current_turn],
	      "      ") ;
      notify_long_message (FALSE, messages[MSG_WORD_NOT_EXISTS][language]) ;
      goto lose_turn ;
    }
  }

  /* Sinon, on vrifie la position des lettres. */
  for (i = 0; i < WORD_LEN; i++) {
    if (current_game.solution[i] ==
	current_game.suggestions_history[current_game.current_turn][i]) {
      seen[i] = TRUE ;
      place[i] = GOOD_PLACE ;
    }
  }

  for (i = 0; i < WORD_LEN; i++) {
    if (place[i] != GOOD_PLACE) {
      for (j = 0; j < WORD_LEN; j++) {
	if ((!seen[j]) &&
	    (current_game.suggestions_history[current_game.current_turn][i] ==
	     current_game.solution[j])) {
	  place[i] = BAD_PLACE ;
	  seen[j] = TRUE ;
	  break ;
	}
      }
    }
  }

 lose_turn:
  for (i = 0; i < WORD_LEN; i++) {
    switch (place[i]) {
    case WRONG_LETTER:
      current_game.colors_history[current_game.current_turn][i] =
	WRONG_LETTER ;
      break ;	/* Fin de WRONG_LETTER. */

    case BAD_PLACE:
      current_game.colors_history[current_game.current_turn][i] =
	BAD_PLACE ;
      break ;	/* Fin de BAD_PLACE. */

    case GOOD_PLACE:
      /* Plaons la lettre sur la ligne suivante c'est--dire, */
      /*  donnons l'indice pour le tour suivant.               */
      current_game.suggestions_history[current_game.current_turn+1][i] =
	current_game.solution[i] ;
      current_game.colors_history[current_game.current_turn+1][i] =
	GOOD_PLACE ;
      /* Maintenant, grons les lettres et couleurs de la ligen courante. */
      current_game.colors_history[current_game.current_turn][i] = GOOD_PLACE ;
      break ;	/* Fin de GOOD_PLACE. */

    default:
      notify_long_message (FALSE, "check_proposition\nASSERT 1") ;
      break ;
    }		/* Fin de switch (place[i]). */
  }		/* Fin de for (i=0; i<WORD_LEN; i++). */

  /* Maintenant, on fusionne les lettres connues des tours prcdents et   */
  /* les lettres que nous venons juste de connatre grce au tour courant. */
  for (i = 0; i < WORD_LEN; i++) {
    if (current_game.current_known_facts[i] != ' ') {
      current_game.suggestions_history[current_game.current_turn+1][i] =
	current_game.current_known_facts[i] ;
      current_game.colors_history[current_game.current_turn+1][i] =
	GOOD_PLACE ;
    }		/* Fin de if (current_known_facts[i] != " "). */
    else {
      if (current_game.colors_history[current_game.current_turn+1][i] ==
	  GOOD_PLACE)
	current_game.current_known_facts[i] =
	  current_game.suggestions_history[current_game.current_turn+1][i] ;
    }		/* Fin de else of if (current_known_facts[i] != " "). */
  }		/* Fin de for (i=0; i<WORD_LEN; i++). */

  /* Tour suivant. */
  current_game.current_turn++ ;
  /* Comme on a remis la premire lettre, c'est comme si */
  /* l'utilisateur avait dj saisi une lettre.          */
  current_game.next_empty_cell = 1 ;
  /* On passe  la case suivante en sautant les */
  /* ventuelles lettres dj bien poses.      */
  while ((current_game.next_empty_cell < WORD_LEN) &&
	 (current_game.colors_history
	  [current_game.current_turn][current_game.next_empty_cell]
	  == GOOD_PLACE))
    current_game.next_empty_cell++ ;
}





void update_game_status (unsigned char entered_pannel_code)
{

  switch (entered_pannel_code) {
  case ORDER_ELAPSED_TIME:
    stop_game_timer () ;
    play_sound (g_wav_elapsed) ; 
    check_proposition (TRUE) ;
    /* Re-dmarage du timer  0. */
    if (current_game.current_turn != -1) {
      clear_timer_frame () ;
      start_game_timer () ;
    }
    refresh_board () ;
    break ;

  case ORDER_VALIDATE_SUGGESTION:
    /* Validation de la saisie de proposition. */
    if (current_game.next_empty_cell == WORD_LEN) {
      stop_game_timer () ;
      check_proposition (FALSE) ;
      /* Re-dmarage du timer  0. */
      if (current_game.current_turn != -1) {
	clear_timer_frame () ;
	start_game_timer () ;
      }
      refresh_board () ;
    }
    else notify_long_message (FALSE, messages[MSG_NOT_6_LETTERS][language]) ;
    break ;

  case ORDER_UNSELECT_LETTER:
    /* Annulation de la dernire lettre saisie. */
    if (current_game.next_empty_cell > 0) {
      play_sound (g_wav_delete) ;
      current_game.next_empty_cell-- ;
      current_game.suggestions_history
	[current_game.current_turn][current_game.next_empty_cell] = ' ' ;
      current_game.colors_history
	[current_game.current_turn][current_game.next_empty_cell] =
	  WRONG_LETTER ;
      refresh_board () ;
    }
    break ;

  case ORDER_INVOKE_PAUSE:
    invoke_pause () ;
    refresh_board () ;
    break ;

  default:
    if (entered_pannel_code < 26) {
      /* Nouvelle lettre saisie. */
      if (current_game.next_empty_cell < WORD_LEN) {
	play_sound (g_wav_insert) ;
	current_game.suggestions_history
	  [current_game.current_turn][current_game.next_empty_cell] =
	  'a' + entered_pannel_code ;
	current_game.colors_history
	  [current_game.current_turn][current_game.next_empty_cell] =
	  JUST_SUBMITTED ;
	/* On passe  la case suivante en sautant les */
	/* ventuelles lettres dj bien poses.      */
	current_game.next_empty_cell++ ;
	while ((current_game.next_empty_cell < WORD_LEN) &&
	       (current_game.colors_history
		[current_game.current_turn][current_game.next_empty_cell]
		== GOOD_PLACE))
	  current_game.next_empty_cell++ ;
	refresh_board () ;
      }
      else
	notify_long_message
	  (FALSE, messages[MSG_ALREADY_6_LETTERS][language]) ;
    }
    else notify_long_message (FALSE, "update_game_status\nASSERT 1") ;
    break ;
  }
}
