/*
  *** PocketMotus GBA ***
 Franois PESSAUX June 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "global.h"
#include "motus_play_fonts.h"
#include "game.h"
#include "background.h"
#include "cursor.h"
#include "stdfonts.h"
#include "utils.h"
#include "intro.h"
#include "requesters.h"
#include "requester_gfx.h"
#include "messages.h"
#include "ints.h"
#include "wav_whistle.h"



#define max(a,b) ((a) > (b) ? (a) : (b))
#define min(a,b) ((a) < (b) ? (a) : (b))


void setup_cursor_graphics (void)
{
  u16 i ;
  u16 *sprites_tiles_memory = SPRITES_CHAR_BASE_BLOCK_ADR ;

  /* Chargement des tiles des sprites. */
  for (i = 0; i < NB_CURSOR_TILES * (32 / 2) ; i++) 
    sprites_tiles_memory[i] =
      (u16) ((cursor_tiles[(i * 2) + 1]<< 8) | (cursor_tiles[(i * 2)])) ;

  /* Chargement de la palette des sprites. */
  for (i = 0; i < NB_CURSOR_PALETTE_ENTRIES ; i++)
    sprite_palette[i] = cursor_palette[i] ;

  /* Cache tous les sprites en dehors de l'ecran pour que */
  /* ceux que l'on n'utilisera pas ne se voient pas.      */
  for (i=0; i<128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
  }

  /* Initialisation du sprite du curseur. */
  cursor_sprite.attribute0 = SPRITE_16_COLOR | SPRITE_SQUARE ;
  cursor_sprite.attribute1 = SPRITE_SIZE_32 ;
  cursor_sprite.attribute2 = CURSOR_SPRITE_TILE_INDEX ;
}





void handle_intro_input ()
{
  u8 cursor_selection, old_cursor_selection ;

  cursor_selection = LANG_FRENCH ;
  /* On attend une pression de touche Start pour continuer. */
  for(;;) {
    old_cursor_selection = cursor_selection ;

    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT(BUTTON_START)) ||
	(KEY_HIT (BUTTON_START))) {
      switch (cursor_selection) {
      case LANG_FRENCH: language = LANG_FRENCH ; return ; break ;
      case LANG_ENGLISH: language = LANG_ENGLISH ; return ; break ;
      case LANG_ITALIANO: language = LANG_ITALIANO ; return ; break ;
      default:
	notify_long_message (FALSE, "handle_intro_input\nASSERT 0") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_LEFT)) {
      /* Dplacement vers la gauche. */
      if (cursor_selection == LANG_FRENCH)
	cursor_selection = LANG_ITALIANO ;
      else cursor_selection-- ;
    }
    
    if (KEY_HIT (KEYPAD_RIGHT)) {
      /* Dplacement vers la droite. */
      cursor_selection = (cursor_selection + 1) % 3 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 x = CURSOR_INTRO_X_FRENCH ;
      switch (cursor_selection) {
      case LANG_FRENCH: x = CURSOR_INTRO_X_FRENCH ; break ;
      case LANG_ENGLISH: x = CURSOR_INTRO_X_ENGLISH ; break ;
      case LANG_ITALIANO: x = CURSOR_INTRO_X_ITALIAN ; break ;
      default:
	notify_long_message (FALSE, "handle_intro_input\nASSERT 1") ;
	break ;
      }
      cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
      cursor_sprite.attribute1 = cursor_sprite.attribute1 | x ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}





void play_intro (void)
{
  u16 i, j, k ;
  u16 *background_tiles_memory = CHAR_BASE_BLOCK_3_ADDR ;

  set_background1 (BACKGROUND_PRIORITY_0 |
		   BACKGROUND_CHAR_BASE_BLOCK_3 |
		   BACKGROUND_COLOR_256 |
		   BACKGROUND_SCREEN_BASE_BLOCK_16 |
		   TEXT_BACKGROUND_256x256) ;

 /* Initialisation de la palette. */
  for (i=0; i<NB_INTRO_PALETTE_ENTRIES; i++)
    palette_memory[i] = intro_palette[i] ;

  /* Chargement des tiles du fond d'cran en mmoire. */
  for (i=0; i<NB_INTRO_TILES * (64 / 2); i++) {
    background_tiles_memory[i] =
      (u16) ((intro_tiles[(i * 2) + 1]<< 8) |
	     (intro_tiles[(i * 2)])) ;
  }

  /* Chargement de la map de fond. */
  k = 0 ;
  for (j=0; j<20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      background_video_memory[(j * (256 / 8)) + i] =
	(u16) intro_map[k] ;
      k++ ;
    }
  }

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  /* Positionnement initial du sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  cursor_sprite.attribute1 = cursor_sprite.attribute1 | CURSOR_INTRO_X_FRENCH ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 | CURSOR_INTRO_Y ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;

  /* Activation de l'affichage. */
  set_screen_mode (MODE_0 | BG1_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_intro_input () ;
  play_sound (g_wav_whistle) ;

  set_screen_mode (MODE_0) ;
}





void setup_board_graphics (void)
{
  u16 i,j,k ;
  u16 *background_tiles_memory = CHAR_BASE_BLOCK_3_ADDR ;
  u16 *balls_tiles_memory = CHAR_BASE_BLOCK_1_ADDR ;
  /* A la suite des tiles de boules. */
  u16 *stdfonts_tiles_memory, *requester_tiles_memory ;


  set_background0 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_1 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_0 | /* Map en mmoire basse. */
		   TEXT_BACKGROUND_256x256) ;       /* 32 * 32 tiles. */
  set_background1 (BACKGROUND_PRIORITY_2 |          /* Pour le fond. */
		   BACKGROUND_CHAR_BASE_BLOCK_3 |   /* Reste 16 Ko pour lui. */
		   BACKGROUND_COLOR_256 |
		   BACKGROUND_SCREEN_BASE_BLOCK_16 |
		   TEXT_BACKGROUND_256x256) ;
  set_background2 (BACKGROUND_PRIORITY_0 |         /* Pour les requesters. */
		   BACKGROUND_CHAR_BASE_BLOCK_1 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_1 |
		   TEXT_BACKGROUND_256x256) ;      /* 32 * 32 tiles. */

 /* Initialisation de la palette. */
  for (i=0; i<NB_BACKGROUND_PALETTE_ENTRIES; i++)
    palette_memory[i] = background_palette[i] ;

  /* Chargement des tiles en mmoire. On crit directement 16 bits dans  */
  /* la mmoire de tiles en 1 coup, donc on va chercher 2 octets dans le */
  /* tableau des donnes  charger (qui lui contient des valeurs de 8    */
  /* bits). En plus, du fait du little endian, il faut inverser les      */
  /* octets de poid fort et de poid faible.                              */
  /* Donc en tout, on itre le nombre de tile * (la taille d'une tile    */
  /* compte en 16 bits). Cette dernire est donc 8 * 8 nibbles = 32,     */
  /* divise par 2. Ouf !                                                */
  for (i=0; i<NB_BALL_FONT_TILES * (32 / 2); i++)
    balls_tiles_memory[i] =
      (u16) ((ball_font_tiles[(i * 2) + 1]<< 8) | (ball_font_tiles[(i * 2)])) ;

  /* On ajoute les tiles des fontes de requester  la suite. */
  stdfonts_tiles_memory = &balls_tiles_memory[i] ;

  for (i = 0; i < NB_STDFONT_TILES * (32 / 2); i++)
    stdfonts_tiles_memory[i] =
      (u16) ((stdfonts_tiles[(i * 2) + 1]<< 8) | (stdfonts_tiles[(i * 2)])) ;

  /* On ajoute les tiles du requester  la suite. */
  requester_tiles_memory = &stdfonts_tiles_memory[i] ;
  for (i = 0; i < NB_REQUESTER_TILES * (32 / 2); i++)
    requester_tiles_memory[i] =
      (u16) ((requester_tiles[(i * 2) + 1] << 8) |
	     (requester_tiles[(i * 2)])) ;

  /* Chargement des tiles du fond d'cran en mmoire. */
  j = 0 ;
  for (i = 0; i < NB_BACKGROUND_TILES * (64 / 2); i++) {
    background_tiles_memory[j] =
      (u16) ((background_tiles[(i * 2) + 1] << 8) |
	     (background_tiles[(i * 2)])) ;
    j++ ;
  }

  /* Chargement des tiles de timer  la suite. */
  for (i = 0; i < NB_TIMER_TILES * (64 / 2); i++) {
    background_tiles_memory[j] =
      (u16) ((timer_tiles[(i * 2) + 1] << 8) |
	     (timer_tiles[(i * 2)])) ;
    j++ ;
  }
 
  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG0HOFS = 0 ;
  REG_BG0VOFS = 0 ;
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  REG_BG2HOFS = 0 ;
  REG_BG2VOFS = 0 ;


  /* Chargement de la map de fond, effacement du tableau de jeu et du */
  /* layer de requester.                                              */
  k = 0 ;
  for (j=0; j<20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      balls_video_memory[(j * (256 / 8)) + i] = (u16) 0 ;
      background_video_memory[(j * (256 / 8)) + i] =
	(u16) background_map[k] ;
      requesters_video_memory[(j * (256 / 8)) + i] =
	(u16) ((requester_map[k] + NB_BALL_FONT_TILES + NB_STDFONT_TILES) |
	       (STD_FONT_SUBPALETTE_NUMBER << 12)) ;
      k++ ;
    }
  }


  /* On dclenche l'affichage seulement une fois que tout est initialis. */
  /* Par dfaut, le layer de requesters n'est pas actif. */
  set_screen_mode (MODE_0 | BG0_ENABLE | BG1_ENABLE | OBJ_MAP_1D |
		   OBJ_ENABLE) ;
}





void refresh_board (void)
{
  u8 i, j ;
  u16 tile_info ;
  u16 tile_on_screen ;

  /* On re-dessine totalement l'espace de jeu. */
  for (i=0; i<NUMBER_OF_TURNS; i++) {
    for (j=0; j<WORD_LEN; j++) {
      /* La couleur de la boule est dtermine par la palette choisie. */
      tile_info = 0 ;
      tile_info |= (current_game.colors_history[i][j] << 12) ;
      /* Les numros de la tile sont dtermins par la mettre  afficher. */
      /* Atttention, chaque boule prend 4 tiles. Si le caractre est ' '  */
      /* alors on prend les 4 premire tiles de la table des caractres.  */
      if (current_game.suggestions_history[i][j] == ' ')
	tile_info |= LETTERS_TILES_OFFSET ;
      else
	/* ... Sinon, on prend les tiles en n'oubliant pas que l'alphabet */
	/* est dcal d'une lettre car la premire sert pour l'espace.    */
	tile_info |=
	  (LETTERS_TILES_OFFSET +
	   (1 + current_game.suggestions_history[i][j] - 'a') * 4) ;
      /* La position est dtermine par les coordonnes de la lettre. */
      tile_on_screen =
	((BOARD_TOP_OFFSET + (i * 2)) * (256 / 8)) /* Offset de ligne. */
	                                     /* 256 : Largeur map en pixels. */
	                                     /* 8 : Nbre pixels par tile. */
	+ 
	(BOARD_LEFT_OFFSET + (j * 2)) ;      /* Offset de colonne. */
      /* Et finalement, on crit les 4 tiles composant la boule. */
      balls_video_memory[tile_on_screen] = tile_info ;
      balls_video_memory[tile_on_screen + 1] = tile_info + 1 ;
      tile_on_screen =
	tile_on_screen + (256 / 8) ;    /* 256 : Largeur map en pixels. */
	                                /* 8 : Nbre pixels par tile. */
      balls_video_memory[tile_on_screen] = tile_info + 2 ;
      balls_video_memory[tile_on_screen + 1] = tile_info + 3 ;
    }
  }
}
