/*
  *** PocketMotus GBA ***
 Franois PESSAUX June 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <gba.h>
#include <sound.h>
#include <interrupt.h>
#include <timer.h>
#include "global.h"
#include "ints.h"
#include "wav_tick.h"



void empty_interrupt_handler ()
{
}


interrupt_handler IntrTable[] = 
{
  empty_interrupt_handler,		/* INT_VBLANK */
  empty_interrupt_handler,		/* INT_HBLANK */
  empty_interrupt_handler,		/* INT_VCOUNT */
  empty_interrupt_handler,		/* INT_TM0 */
  empty_interrupt_handler,		/* INT_TM1 */
  empty_interrupt_handler,		/* INT_TM2 */
  empty_interrupt_handler,		/* INT_TM3 */
  empty_interrupt_handler,		/* INT_COM */
  empty_interrupt_handler,		/* INT_DMA0 */
  empty_interrupt_handler,		/* INT_DMA1 */
  empty_interrupt_handler,		/* INT_DMA2 */
  empty_interrupt_handler,		/* INT_DMA3 */
  empty_interrupt_handler,		/* INT_KEYS */
  empty_interrupt_handler,		/* INT_CART */
};



void timer1_sound_completed_handler ()
{
  REG_DMA1CNT_H = 0 ;  /* Stop la DMA 1. */
  REG_TM0CNT_H = 0 ;   /* Dsactive le timer 0. */
  REG_TM1CNT_H = 0 ;   /* Dsactive le timer 1. */

  /* Positionner le flag pour dire que l'on a trait l'interruption. */
  MARK_IT_AS_HANDLED (INTERRUPT_TIMER1) ;
}


void timer2_play_tick_handler ()
{
  play_sound (g_wav_tick) ;
  MARK_IT_AS_HANDLED (INTERRUPT_TIMER2) ;
}


#define SND_FREQUENCY ((u16) 8000)


void init_sound_handler ()
{
  IntrTable[INTERRUPT_VECTOR_TM1_INDEX] =
    (interrupt_handler) timer1_sound_completed_handler ;
  /* Autoriser la gestion de l'interruption d'overflow du timer 1. */
  SET_INTERRUPT (INT_TIMER1) ;
  IntrTable[INTERRUPT_VECTOR_TM2_INDEX] =
    (interrupt_handler) timer2_play_tick_handler ;
  /* Autoriser la gestion de l'interruption d'overflow du timer 2. */
  SET_INTERRUPT (INT_TIMER2) ;
  ENABLE_MASTER_INTERRUPT ;
}






void play_sound (const u32 *snd_data)
{
  REG_DMA1CNT_H = 0 ;   /* Stop la DMA 1. */
  REG_TM0CNT_H = 0 ;    /* Dsactive le timer 0. */
  REG_TM1CNT_H = 0 ;    /* Dsactive le timer 1. */

  /* On nettoie le registre de contrle du son. */
  REG_SOUNDCNT_L = 0;

  REG_SOUNDCNT_H = SND_OUTPUT_RATIO_100 | DSA_OUTPUT_RATIO_100 |
    DSA_OUTPUT_TO_BOTH | DSA_TIMER0 | DSA_FIFO_RESET ;

  REG_SOUNDCNT_X = SND_ENABLED ;

  REG_DMA1SAD = (u32) (&snd_data[1]) ;
  REG_DMA1DAD = (u32) REG_SGFIFOA_ADDR ;
  REG_DMA1CNT = ENABLE_DMA | START_ON_FIFO_EMPTY | WORD_DMA | DMA_REPEAT ;

  /* On utilise le timer 1 pour compter le nombre de samples que l'on   */
  /* a jou. Donc on l'initialise  0xFFFF - le nombre d'octets du son. */
  REG_TM1CNT_L = (u16) (0xFFFF - snd_data[0]) ;

  /* On utilise le timer 0 comme rgulateur du sampling. */
  REG_TM0CNT_L = (u16) (0xFFFF - (CPU_FREQUENCY / SND_FREQUENCY)) ;
  REG_TM1CNT_H = TIMER_CASCADING | TIMER_OVERFLOW_INTERRUPT | TIMER_ON_BIT ;
  REG_TM0CNT_H = TIMER_FREQ_16_78_MHZ | TIMER_ON_BIT ;
}
