/*
  *** PocketMotus GBA ***
 Franois PESSAUX June 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <timer.h>
#include "global.h"
#include "game.h"
#include "pause_gfx.h"
#include "requesters.h"
#include "messages.h"
#include "cursor.h"
#include "graphics.h"
#include "utils.h"
#include "background.h"



void handle_pause_input ()
{
  u8 cursor_selection, old_cursor_selection ;

  cursor_selection = SELECTION_CONTINUE ;
  for(;;) {
    old_cursor_selection = cursor_selection ;

    /* Le bouton B fait reprendre par dfaut. */
    KEY_POOL ;
    if (KEY_HIT (BUTTON_B)) break ;

    if (KEY_HIT (BUTTON_A)) {
      switch (cursor_selection) {
      case SELECTION_CONTINUE: return ;	break ;

      case SELECTION_SURRENDER:
	if (current_game.current_turn > -1) {
	  /* Abandon du jeu courant. */
	  if (notify_long_message
	      (TRUE, messages[MSG_ABANDON_QUESTION][language])) {
	    strcpy
	      (all_purpose_message_buffer,
	       messages[MSG_ADANDON_SOLUTION][language]) ;
	    strcat (all_purpose_message_buffer, current_game.solution) ;
	    notify_long_message (FALSE, all_purpose_message_buffer) ;
	    TerminateGame (SURRENDER) ;
	    return ;
	  }
	}
	break ;

      default:
	notify_long_message (FALSE, "handle_pause_input\nASSERT 1") ;
	break ;
      }
    }

    if ((KEY_HIT (KEYPAD_UP)) || (KEY_HIT (KEYPAD_DOWN))) {
      /* Dplacement vers le haut ou vers le bas. */
      if (cursor_selection == SELECTION_CONTINUE)
	cursor_selection = SELECTION_SURRENDER ;
      else cursor_selection = SELECTION_CONTINUE ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 y = PAUSE_OPTIONS_CONTINUE_Y_OFFSET ;
      switch (cursor_selection) {
      case SELECTION_CONTINUE: y = PAUSE_OPTIONS_CONTINUE_Y_OFFSET ; break ;
      case SELECTION_SURRENDER: y = PAUSE_OPTIONS_SURRENDER_Y_OFFSET ; break ;
      default:
	notify_long_message (FALSE, "handle_pause_input\nASSERT 2") ;
	break ;
      }
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}





void invoke_pause (void)
{
  short i, j, k ;
  u16 *background_tiles_memory = CHAR_BASE_BLOCK_3_ADDR ;

  /* On dsactive le timer en mettant sont bit de cascade. */
  SUSPEND_GAME_TIMER ;

  /* On met l'cran an blackout. */
  set_screen_mode (0) ;

  set_background1 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_3 |
		   BACKGROUND_COLOR_256 |
		   BACKGROUND_SCREEN_BASE_BLOCK_16 |
		   TEXT_BACKGROUND_256x256) ;

 /* Initialisation de la palette. */
  for (i = 0; i < NB_PAUSE_PALETTE_ENTRIES; i++)
    palette_memory[i] = pause_palette[i] ;

  /* Chargement des tiles du fond d'cran en mmoire. */
  for (i = 0; i < NB_PAUSE_TILES * (64 / 2); i++) {
    background_tiles_memory[i] =
      (u16) ((pause_tiles[(i * 2) + 1] << 8) |
	     (pause_tiles[(i * 2)])) ;
  }

  /* Chargement de la map de fond. */
  k = 0 ;
  for (j = 0; j < 20; j++) {          /* Boucle verticale. */
    for (i = 0; i < 30; i++) {        /* Boucle honrizontale. */
      background_video_memory[(j * (256 / 8)) + i] =
	(u16) pause_map[k] ;
      k++ ;
    }
  }

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  /* Positionnement initial du sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  cursor_sprite.attribute1 =
    cursor_sprite.attribute1 | PAUSE_OPTIONS_RIGHT_OFFSET ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 =
    cursor_sprite.attribute0 | PAUSE_OPTIONS_CONTINUE_Y_OFFSET ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
  /* Activation de l'affichage. */
  set_screen_mode (MODE_0 | BG1_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_pause_input () ;

  /* On met l'cran an blackout. */
  set_screen_mode (0) ;

  /* On raffrachit l'cran de jeu et on restaure le mode vido d'origine. */
  setup_board_graphics () ;

  /* On rafrachit le timer si ncessaire. */
  j = (REG_TM3CNT_L / 10) - 1 ;
  for (i = 0; i <= j; i++) draw_timer_tick (TIMER_FILLED_TILE, i) ;

  /* On ractive le timer en supprimant son bit de cascade. */
  RELAUNCH_GAME_TIMER ;
}
