/*
  *** PocketMotus GBA ***
 Franois PESSAUX June 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "global.h"
#include "game.h"
#include "starter_gfx.h"
#include "requesters.h"
#include "messages.h"
#include "cursor.h"
#include "graphics.h"
#include "utils.h"
#include "ints.h"
#include "wav_ready.h"




void display_flag (u8 y_offset, u16 start_from_tile)
{
  u16 tile_on_screen ;

  tile_on_screen = (y_offset * (256 / 8)) + STARTER_FLAGS_RIGHT_OFFSET ;
  background_video_memory[tile_on_screen] = start_from_tile ;
  tile_on_screen++ ;
  background_video_memory[tile_on_screen] = start_from_tile + 1 ;
  tile_on_screen = ((y_offset + 1) * (256 / 8)) + STARTER_FLAGS_RIGHT_OFFSET ;
  background_video_memory[tile_on_screen] = start_from_tile + 2 ;
  tile_on_screen++ ;
  background_video_memory[tile_on_screen] = start_from_tile + 3 ;
}



void handle_starter_input ()
{
  u8 cursor_selection, old_cursor_selection ;
  u16 tmp_tile_start ;

  cursor_selection = SELECTION_START ;
  for(;;) {
    old_cursor_selection = cursor_selection ;

    KEY_POOL ;
    /* Le bouton B fait comencer par dfaut. */
    if (KEY_HIT (BUTTON_B)) break ;

    if (KEY_HIT (BUTTON_A)) {
      switch (cursor_selection) {
      case SELECTION_START: return ; break ;

      case SELECTION_CHECK_WORD:
	check_flag = !check_flag ;
	tmp_tile_start  = (check_flag ? FLAG_YES_TILE : FLAG_NO_TILE) ;
	display_flag
	  (STARTER_OPTIONS_CHECK_WORD_FLAG_Y_OFFSET, tmp_tile_start) ;
	break ;

      case SELECTION_RESTRICTED_TIME:
	timer_flag = !timer_flag ;
	tmp_tile_start  = (timer_flag ? FLAG_YES_TILE : FLAG_NO_TILE) ;
	display_flag
	  (STARTER_OPTIONS_RESTRICTED_TIME_FLAG_Y_OFFSET, tmp_tile_start) ;
	break ;

      case SELECTION_SAVE_STATS: SaveGame () ; break ;

      case SELECTION_CLEAR_STATS: ClearStats () ; break ;

      default:
	notify_long_message (FALSE, "handle_starter_input\nASSERT 1") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_UP)) {
      /* Dplacement vers le haut. */
      if (cursor_selection == SELECTION_START)
	cursor_selection = SELECTION_CLEAR_STATS ;
      else cursor_selection-- ;
    }

    if (KEY_HIT (KEYPAD_DOWN)) {
      /* Dplacement vers le bas. */
      cursor_selection = (cursor_selection + 1) % 5 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 y = STARTER_OPTIONS_START_Y_OFFSET ;
      switch (cursor_selection) {
      case SELECTION_START: y = STARTER_OPTIONS_START_Y_OFFSET ; break ;
      case SELECTION_CHECK_WORD: y = STARTER_OPTIONS_CHECK_WORD_Y_OFFSET ;
	break ;
      case SELECTION_RESTRICTED_TIME :
	y = STARTER_OPTIONS_RESTRICTED_TIME_Y_OFFSET ;
	break ;
      case SELECTION_SAVE_STATS:
	y = STARTER_OPTIONS_SAVE_STATS_Y_OFFSET ;
	break ;
      case SELECTION_CLEAR_STATS:
	y = STARTER_OPTIONS_CLEAR_STATS_Y_OFFSET ;
	break ;
      default:
	notify_long_message (FALSE, "handle_starter_input\nASSERT 2") ;
	break ;
      }
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}




void invoke_starter (void)
{
  u16 i, j, k ;
  u16 tile_index ;
  u16 *background_tiles_memory = CHAR_BASE_BLOCK_3_ADDR ;
  u16 tmp_tile_start ;

  /* On met l'cran an blackout. */
  set_screen_mode (0) ;

  set_background1 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_3 |
		   BACKGROUND_COLOR_256 |
		   BACKGROUND_SCREEN_BASE_BLOCK_16 |
		   TEXT_BACKGROUND_256x256) ;

 /* Initialisation de la palette. */
  for (i = 0; i < NB_STARTER_PALETTE_ENTRIES; i++)
    palette_memory[i] = starter_palette[i] ;

  tile_index = 0 ;
  /* Chargement des tiles du fond d'cran en mmoire. */
  for (i = 0; i < NB_STARTER_TILES * (64 / 2); i++) {
    background_tiles_memory[tile_index] =
      (u16) ((starter_tiles[(i * 2) + 1] << 8) |
	     (starter_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de flag yes en mmoire. */
  for (i = 0; i < NB_STARTER_FLAG_TILES_PER_FLAG * (64 / 2); i++) {
    background_tiles_memory[tile_index] =
      (u16) ((starter_flag_yes_tiles[(i * 2) + 1] << 8) |
	     (starter_flag_yes_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de flag no en mmoire. */
  for (i = 0; i < NB_STARTER_FLAG_TILES_PER_FLAG * (64 / 2); i++) {
    background_tiles_memory[tile_index] =
      (u16) ((starter_flag_no_tiles[(i * 2) + 1] << 8) |
	     (starter_flag_no_tiles[(i * 2)])) ;
    tile_index++ ;
  }


  /* Chargement de la map de fond. */
  k = 0 ;
  for (j = 0; j < 20; j++) {          /* Boucle verticale. */
    for (i = 0; i < 30; i++) {        /* Boucle honrizontale. */
      background_video_memory[(j * (256 / 8)) + i] =
	(u16) starter_map[k] ;
      k++ ;
    }
  }

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  /* Positionnement initial du sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  cursor_sprite.attribute1 =
    cursor_sprite.attribute1 | STARTER_OPTIONS_RIGHT_OFFSET ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 =
    cursor_sprite.attribute0 | STARTER_OPTIONS_START_Y_OFFSET ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;

  /* On positionne les icones de flag initialement. */
  tmp_tile_start  = (check_flag ? FLAG_YES_TILE : FLAG_NO_TILE) ;
  display_flag
    (STARTER_OPTIONS_CHECK_WORD_FLAG_Y_OFFSET, tmp_tile_start) ;
  tmp_tile_start  = (timer_flag ? FLAG_YES_TILE : FLAG_NO_TILE) ;
  display_flag
    (STARTER_OPTIONS_RESTRICTED_TIME_FLAG_Y_OFFSET, tmp_tile_start) ;


  /* Activation de l'affichage. */
  set_screen_mode (MODE_0 | BG1_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_starter_input () ;
  play_sound (g_wav_ready) ;

  /* On met l'cran an blackout. */
  set_screen_mode (0) ;

  /* On raffrachit l'cran de jeu et on restaure le mode vido d'origine. */
  setup_board_graphics () ;
}
