/*
  *** PocketMotus GBA ***
 Franois PESSAUX June 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include "global.h"
#include "game.h"
#include <timer.h>
#include "cursor.h"
#include "background.h"




short prev_drawn_tick = -1 ;



void draw_timer_tick (u16 with_tile, short tick)
{
  u16 tile_info, tile_on_screen ;

  /* Pour viter les overflows si on n'a pas lu le timer  temps. */
  if ((tick < 0) || (tick >= 6)) return ;

  tile_on_screen = ((TIMER_TOP_OFFSET) * (256 / 8)) +
    (TIMER_LEFT_OFFSET + (tick * 2)) ;
  tile_info = with_tile ;
  background_video_memory[tile_on_screen] = tile_info ;
  tile_on_screen++ ;
  background_video_memory[tile_on_screen] = tile_info + 1 ;
  tile_on_screen = ((TIMER_TOP_OFFSET + 1) * (256 / 8)) +
    (TIMER_LEFT_OFFSET + (tick * 2)) ;
  background_video_memory[tile_on_screen] = tile_info + 2 ;
  tile_on_screen++ ;
  background_video_memory[tile_on_screen] = tile_info + 3 ;
}





void clear_timer_frame ()
{
  u8 i ;

  for (i = 0; i < 6; i++) draw_timer_tick (TIMER_EMPTY_TILE, i) ;
}





unsigned char read_user_input_in_game_state (void)
{
  static u8 cursor_x = INPUT_LEFT_OFFSET ;
  static u8 cursor_y = INPUT_TOP_OFFSET ;
  BOOL update_cursor = FALSE ;
  short current_tick ;

  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  /* On fait -13 car la forme du curseur a une    */
  /* pointe dcale de 13 pixels horizontalement. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 | ((cursor_x * 8) - 13) ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 | ((cursor_y * 8) + 5) ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;

  for (;;) {
    /* Gestion du temps. */
    if (timer_flag) {
      /* Mise  jour de la progression graphique du timer. */
      current_tick = (REG_TM3CNT_L / 10) - 1 ;
      if (prev_drawn_tick < current_tick) {
	draw_timer_tick (TIMER_FILLED_TILE, current_tick) ;
	prev_drawn_tick = current_tick ;
      }
      /* Vrification du temps imparti. */
      if (REG_TM3CNT_L >= 60) {
	/* On dessine le dernier slot du timer. */
	draw_timer_tick (TIMER_FILLED_TILE, 5) ;
	return (ORDER_ELAPSED_TIME) ;
      }
    }

    KEY_POOL ;
    if (KEY_HIT (BUTTON_SELECT)) {
      /* Suggestion valide. */
      return (ORDER_INVOKE_PAUSE) ;
    }

    if (KEY_HIT (BUTTON_START)) {
      /* Partie abandonne. */
      return (ORDER_VALIDATE_SUGGESTION) ;
    }

    if (KEY_HIT (BUTTON_A)) {
      /* Lettre slectionne. */
      return ((7 * (cursor_y - INPUT_TOP_OFFSET)) +
	      (cursor_x - INPUT_LEFT_OFFSET)) ;
    }
    if (KEY_HIT (BUTTON_B)) {
      /* Lettre d-slctionne. */
      return (ORDER_UNSELECT_LETTER) ;
    }

    if (KEY_HIT (KEYPAD_LEFT)) {
      /* Dplacement vers la gauche. */
      if (cursor_x > INPUT_LEFT_OFFSET) {
	cursor_x-- ;
	update_cursor = TRUE ;
      }
    }

    if (KEY_HIT (KEYPAD_RIGHT)) {
      /* Dplacement vers la droite. */
      if (cursor_x < INPUT_RIGHT_OFFSET) {
	cursor_x++ ;
	update_cursor = TRUE ;
      }
    }

    if (KEY_HIT (KEYPAD_UP)) {
      /* Dplacement vers le haut. */
      if (cursor_y > INPUT_TOP_OFFSET) {
	cursor_y-- ;
	update_cursor = TRUE ;
      }
    }

    if (KEY_HIT (KEYPAD_DOWN)) {
      /* Dplacement vers le bas. */
      if (cursor_y < INPUT_BOTTOM_OFFSET) {
	cursor_y++ ;
	update_cursor = TRUE ;
      }
    }

    if (update_cursor == TRUE) {
      /* Efface l'ancienne abscisse. */
      cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
      /* On fait -13 car la forme du curseur a une    */
      /* pointe dcale de 13 pixels horizontalement. */
      cursor_sprite.attribute1 =
	cursor_sprite.attribute1 | ((cursor_x * 8) - 13) ;
      /* Efface l'ancienne ordonne. */
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      /* Mme genre de correction pour l'ordonne. */
      cursor_sprite.attribute0 =
	cursor_sprite.attribute0 | ((cursor_y * 8) + 5) ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
      update_cursor = FALSE ;
    }
  }
}




void itoa (int x, unsigned char *buffer)
{
  u8 i = 0 ;
  u8 j ;
  unsigned char tmp_c ;
  int div = x ;
  int remain = 0 ;

  if (x == 0) {
    buffer[0] = '0' ;
    buffer[1] = '\0' ;
  }
  else
    {
      while (div != 0) {
	remain = div % 10 ;
	div = div / 10 ;
	buffer [i] = '0' + remain ;
	i++ ;
      }
      buffer[i] = '\0' ;
      i-- ;

      /* Inversion de la chane. */
      for (j = i; i > j / 2; i--) {
	tmp_c = buffer[i] ;
	buffer[i] = buffer[j-i] ;
	buffer[j-i] = tmp_c ;
      }
    }
}





void stop_game_timer ()
{
  if (timer_flag) {
    /* Arrt du timer. */
    SET_TIMER_OFF (REG_TM2CNT_H) ;
    SET_TIMER_OFF (REG_TM3CNT_H) ;
  }
}





void start_game_timer ()
{
  if (timer_flag) {
    prev_drawn_tick = -1 ;
    REG_TM2CNT_L = (u16) 0 ;
    REG_TM3CNT_L = (u16) 0 ;
    /* Frquence pour 0xFFFF correspondant  1 s. */
    REG_TM2CNT_H =
      TIMER_FREQ_65_5_KHZ | TIMER_ON_BIT | TIMER_OVERFLOW_INTERRUPT ;
    /* Timer en cascade pour tester 60 overflows du timer 2 (=1 mn). */
    REG_TM3CNT_H = TIMER_CASCADING | TIMER_ON_BIT ;
  }
}
